package nl.pim16aap2.armoredElytra.nbtEditor.ArmoredElytraBuilder;

import nl.pim16aap2.armoredElytra.util.ConfigLoader;
import org.bukkit.Color;
import org.bukkit.inventory.ItemStack;

import javax.annotation.Nullable;
import java.util.List;

/**
 * An interface describing the last step of building an armored elytra: updating metadata
 */
public interface ItemMetadataSetter {

    /**
     * Specifies the new name of the armored elytra.
     *
     * @param name The new name of the armored elytra. When this is null (default), the default name for the new
     *             tier will be used.
     * @return <p>The current metadata setter.</p>
     */
    ItemMetadataSetter withName(@Nullable String name);

    /**
     * Specifies the new color of the armored elytra.
     *
     * @param color <p>The new color of the armored elytra. When this is null (default), the color to use is inferred
     *              from the creation process.</p>
     * @return <p>The current metadata setter.</p>
     */
    ItemMetadataSetter withColor(@Nullable Color color);

    /**
     * Specifies the texture identifier (custom_model_data) for the armored elytra
     *
     * @param textureId <p>The texture id to set for the armored elytra. If null, the default values will be used.</p>
     * @return <p>The current metadata setter.</p>
     */
    ItemMetadataSetter withTexture(@Nullable Integer textureId);

    /**
     * Specifies the new lore of the armored elytra.
     *
     * @param lore The new lore of the armored elytra. When this is null (default), the default lore for the new
     *             tier will be used.
     * @return <p>The current metadata setter.</p>
     */
    //TODO: UNUSED!
    ItemMetadataSetter withLore(@Nullable List<String> lore);

    /**
     * Specifies whether the armored elytra should be unbreakable.
     * <p>
     * By default, this value is read from {@link ConfigLoader#unbreakable()}.
     *
     * @param isUnbreakable True if the armored elytra should be unbreakable.
     * @return <p>The current metadata setter.</p>
     */
    //TODO: UNUSED!
    ItemMetadataSetter unbreakable(boolean isUnbreakable);

    /**
     * Constructs the armored elytra from the provided configuration.
     *
     * @return <p>The new armored elytra.</p>
     */
    ItemStack build();

}
