package nl.pim16aap2.armoredElytra.util;

import org.bukkit.Material;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;

import javax.annotation.Nullable;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Locale;
import java.util.Map;

public class Util {
    public static String errorToString(Error e) {
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw));
        return sw.toString();
    }

    public static String exceptionToString(Exception e) {
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw));
        return sw.toString();
    }

    // Check if an item is broken or not.
    public static boolean isBroken(ItemStack item) {
        return item.getDurability() >= item.getType().getMaxDurability();
    }

    // Get the armor tier from a chest plate.
    public static ArmorTier armorToTier(ItemStack itemStack) {
        return armorToTier(itemStack.getType());
    }

    // Get the armor tier from a chest plate.
    public static ArmorTier armorToTier(Material material) {
        ArmorTier armorTier = ArmorTier.NONE;

        switch (material) {
            case LEATHER_CHESTPLATE -> armorTier = ArmorTier.LEATHER;
            case GOLDEN_CHESTPLATE -> armorTier = ArmorTier.GOLD;
            case CHAINMAIL_CHESTPLATE -> armorTier = ArmorTier.CHAIN;
            case IRON_CHESTPLATE -> armorTier = ArmorTier.IRON;
            case DIAMOND_CHESTPLATE -> armorTier = ArmorTier.DIAMOND;
            case NETHERITE_CHESTPLATE -> armorTier = ArmorTier.NETHERITE;
            default -> {
            }
        }
        return armorTier;
    }

    public static boolean isChestPlate(ItemStack itemStack) {
        return isChestPlate(itemStack.getType());
    }

    // Check if mat is a chest plate.
    public static boolean isChestPlate(Material material) {
        return material == Material.LEATHER_CHESTPLATE || material == Material.GOLDEN_CHESTPLATE ||
                material == Material.CHAINMAIL_CHESTPLATE || material == Material.IRON_CHESTPLATE ||
                material == Material.DIAMOND_CHESTPLATE || material == Material.NETHERITE_CHESTPLATE;
    }

    public static String snakeToCamelCase(String input) {
        final char[] arr = input.toLowerCase(Locale.US).toCharArray();

        int skipped = 0;
        boolean capitalize = false;

        for (int idx = 0; idx < arr.length; ++idx) {
            char current = arr[idx];

            if (current == '_') {
                ++skipped;
                capitalize = true;
                continue;
            }

            final int targetIdx = idx - skipped;

            if (capitalize) {
                if (targetIdx > 0) {
                    current = Character.toUpperCase(current);
                }
                capitalize = false;
            }

            // targetIdx is always <= idx, so we can reuse the current array
            // without overwriting any values we will need in the future.
            arr[targetIdx] = current;
        }

        return new String(arr, 0, arr.length - skipped);
    }

    // Function that returns which/how many protection enchantments there are.
    public static int getProtectionEnchantmentsVal(Map<Enchantment, Integer> enchantments) {
        int ret = 0;
        if (enchantments.containsKey(Enchantment.PROTECTION_ENVIRONMENTAL)) {
            ret += 1;
        }
        if (enchantments.containsKey(Enchantment.PROTECTION_EXPLOSIONS)) {
            ret += 2;
        }
        if (enchantments.containsKey(Enchantment.PROTECTION_FALL)) {
            ret += 4;
        }
        if (enchantments.containsKey(Enchantment.PROTECTION_FIRE)) {
            ret += 8;
        }
        if (enchantments.containsKey(Enchantment.PROTECTION_PROJECTILE)) {
            ret += 16;
        }
        return ret;
    }

    public static void moveChestplateToInventory(Player player) {
        final PlayerInventory inventory = player.getInventory();
        inventory.addItem(inventory.getChestplate());

        final @Nullable ItemStack chestplate = inventory.getChestplate();
        if (chestplate != null) {
            chestplate.setAmount(0);
        }

        player.updateInventory();
    }

    /**
     * Ensures that a given value does not exceed the provided upper and lower bounds.
     *
     * @param val The value to check.
     * @param min The lower bound limit.
     * @param max The upper bound limit.
     * @return The value if it is bigger than min and larger than max, otherwise either min or max.
     */
    public static int between(int val, int min, int max) {
        return Math.max(min, Math.min(max, val));
    }

    /**
     * Gets the correct texture id from the given armor tier
     *
     * @param armorTier <p>The armor tier to get texture for</p>
     * @return <p>The texture to be used for the given armor tier</p>
     */
    public static Integer getTextureIdFromArmorTier(ArmorTier armorTier) {
        return switch (armorTier) {
            case NONE -> null;
            case LEATHER -> 471301;
            case GOLD -> 471302;
            case CHAIN -> 471303;
            case IRON -> 471304;
            case DIAMOND -> 471305;
            case NETHERITE -> 471306;
        };
    }

}
